/*
*  pgpcache.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "PgpCache.h"
#include "qsdbs.h"
#include "gpg.h"
#include "utility.h"
//---------------------------------------------------------------------------
TPGPPassphraseCache *PGPCache;
const DWORD MaxDWORD = 4294967295;
//---------------------------------------------------------------------------
// TPhrase
//---------------------------------------------------------------------------

__fastcall TPGPPassphrase::TPGPPassphrase(const AnsiString &PGPIDText8,
   AnsiString &Passphrase, int Minutes) : TObject()
{
   AnsiString key;
   DB->LoadSessionKey(key);
   encryptedphrase = new TMemoryStream();
   GPGEncryptStringToStream(key, Passphrase, encryptedphrase);
   idtext8 = PGPIDText8;
   SetExpire(Minutes);
}
//---------------------------------------------------------------------------

__fastcall TPGPPassphrase::~TPGPPassphrase()
{
   ZeroMemory(encryptedphrase->Memory, encryptedphrase->Size);
   delete encryptedphrase;
}
//---------------------------------------------------------------------------

void __fastcall TPGPPassphrase::SetExpire(int minutes)
{
   starttick = GetTickCount();
   double expiretick = starttick + (minutes * 60000);
   lasttick = (expiretick > MaxDWORD ? expiretick - MaxDWORD : expiretick);
}
//---------------------------------------------------------------------------

bool __fastcall TPGPPassphrase::Expired()
{
   DWORD curtick = GetTickCount();
   return (lasttick <= curtick) &&
      ((lasttick > starttick) || (curtick < starttick));
}

//---------------------------------------------------------------------------
// TPGPPassphraseCache
//---------------------------------------------------------------------------

__fastcall TPGPPassphraseCache::TPGPPassphraseCache()
{
   timer = new TTimer(NULL);
   timer->Enabled = false;
   timer->Interval = 10000;  // 10 seconds
   timer->OnTimer = OnTimer;
   passphrases = new TList();
}
//---------------------------------------------------------------------------

__fastcall TPGPPassphraseCache::~TPGPPassphraseCache()
{
   Clear();
   delete passphrases;
   delete timer;
}
//---------------------------------------------------------------------------

void __fastcall TPGPPassphraseCache::Add(const AnsiString &idtext8, AnsiString &passphrase)
{
   assert(passphrase != "");
   int minutes = DB->GetConfigItem(PGPConfig, GPGcacheminutes, GPGCacheMinutes);
   if (minutes == 0)
   {
      ZeroString(passphrase);
      return;
   }

   timer->Enabled = false;
   passphrases->Add(new TPGPPassphrase(idtext8, passphrase, minutes));
   ZeroString(passphrase);
   timer->Enabled = true;
}
//---------------------------------------------------------------------------

bool __fastcall TPGPPassphraseCache::Get(TStrings *idtext8s, AnsiString &idtext8,
   AnsiString &passphrase) // returns idtext8 and it's phrase
{
   bool got = false;
   timer->Enabled = false;
   for (int i = 0; i < passphrases->Count; i++)
   {
      TPGPPassphrase *phrase = GetPGPPassphrase(i);
      if (idtext8s->IndexOf(phrase->idtext8) >= 0)
      {
         AnsiString key;
         DB->LoadSessionKey(key);
         GPGDecryptStreamToString(key, phrase->encryptedphrase, passphrase);
         idtext8 = phrase->idtext8;
         delete phrase;
         passphrases->Delete(i);
         got = true;
         break;
      }
   }
   timer->Enabled = true;
   return got;
}
//---------------------------------------------------------------------------

bool __fastcall TPGPPassphraseCache::Get(const AnsiString &idtext8, AnsiString &passphrase)
{
   bool enabled = timer->Enabled;
   if (enabled)
      timer->Enabled = false;

   passphrase = "";
   bool got = false;
   for (int i = 0; i < passphrases->Count; i++)
   {
      TPGPPassphrase *phrase = GetPGPPassphrase(i);
      if (phrase->idtext8 == idtext8)
      {
         AnsiString key;
         DB->LoadSessionKey(key);
         GPGDecryptStreamToString(key, phrase->encryptedphrase, passphrase);
         phrase->SetExpire(DB->GetConfigItem(PGPConfig, GPGcacheminutes, GPGCacheMinutes));
         got = true;
         break;
      }
   }

   if (enabled)
      timer->Enabled = true;
   return got;
}
//---------------------------------------------------------------------------

TPGPPassphrase* __fastcall TPGPPassphraseCache::GetPGPPassphrase(int index)
{
   assert(index >= 0);
   assert(index < passphrases->Count);
   return (TPGPPassphrase*)passphrases->Items[index];
}
//---------------------------------------------------------------------------

void __fastcall TPGPPassphraseCache::OnTimer(TObject *Sender)
{
   timer->Enabled = false;
   int i = 0; while (i < passphrases->Count)
   {
      TPGPPassphrase *phrase = GetPGPPassphrase(i);
      if (phrase->Expired())
      {
         delete phrase;
         passphrases->Delete(i);
         continue;
      }
      i++;
   }
   if (passphrases->Count)
      timer->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TPGPPassphraseCache::Clear()
{
   timer->Enabled = false;
   while (passphrases->Count)
   {
      delete GetPGPPassphrase(0);
      passphrases->Delete(0);
   }
}
//---------------------------------------------------------------------------

int __fastcall TPGPPassphraseCache::Count()
{
   return passphrases->Count;
}
//---------------------------------------------------------------------------

